proc format ;
  value misf 1="Missing"
             0="Observed" ;
run ;

data test_mnar_comp ;
  call streaminit(777) ;

  mux=0 ; muy=0 ;
  sx =1 ; sy =1 ;
  rho=0.3 ;

  do i=1 to 100 ;
    x=rand("normal",mux,sx) ;
    y=rand("normal",muy+rho*sy/sx*(x-mux),sqrt((1-rho**2)*sy**2)) ;
    output ;
  end ;
run ;

data test_mnar_miss ;
  set test_mnar_comp ;

  call streaminit(77777) ;

  a=-10 ; b=15 ; c=20 ;

  p=exp(a+b*x+c*y)/(1+exp(a+b*x+c*y)) ;
  miss=rand("bernoulli",p) ;

  if miss=1 then y_o=. ;
  else y_o=y ;
run ;

proc corr data=test_mnar_miss ;
  var x y y_o ;
run ;

proc sort data=test_mnar_miss ;
  by miss ;
run ;

proc sgplot data=test_mnar_miss ;
  scatter x=x y=y / group=miss ;
  xaxis values=(-3 to 3) ;
  yaxis values=(-3 to 3) ;
  format miss misf. ;
run ;

proc sgplot data=test_mnar_miss ;
  where miss=0 ;
  scatter x=x y=y / group=miss ;
  xaxis values=(-3 to 3) ;
  yaxis values=(-3 to 3) ;
  format miss misf. ;
run ;

proc iml ;
  use test_mnar_miss ;
  read all var {miss x y_o} into data ;

  obs=loc(data[,1]=0) ;
  mis=loc(data[,1]=1) ;

  x_obs=data[,2][obs] ; * yϑꂽR[hx̒l܂ރxNg ;
  y_obs=data[,3][obs] ; * yϑꂽR[hy̒l܂ރxNg ;
  x_mis=data[,2][mis] ; * ỹR[hx̒l܂ރxNg ;

  n_obs=nrow(x_obs) ; * yϑꂽR[h ;
  n_mis=nrow(x_mis) ; * ỹR[h ;

  start integral(y) global(x_mis, psi, ecurr, vcurr, i) ;
    pi=constant("PI") ;

    ex=exp(psi[1]+psi[2]#x_mis[i]+psi[3]#y) ;
    p=ex/(1+ex) ;

    int = p#(1/(sqrt(2#pi#vcurr)))#exp(-(1/2)#(y-ecurr)##2/vcurr) ;
    integral = int[i] ;

    return (integral) ;
  finish integral ;

  start likelihood(param) global(x_obs, y_obs, x_mis, psi, ecurr, vcurr, n_mis, i) ;
                 * param = (mux muy sx sy rho psi1-psi3): 肷p[^ ;

    theta = param[1:5] ;
    psi   = param[6:8] ;

    * x,yƂɊϑ ;
    ll_obs_1 = -log(theta[3]) -log(theta[4]) -1/2#log(1-theta[5]##2)
               -1/(2#(1-theta[5]##2)) # ( (x_obs-theta[1])##2/theta[3]##2 -2#theta[5]#(x_obs-theta[1])#(y_obs-theta[2])/(theta[3]#theta[4])
                +(y_obs-theta[2])##2/theta[4]##2 ) ;
    ll_obs_2 = -log(1+exp(psi[1]+psi[2]#x_obs+psi[3]#y_obs)) ;

    * xϑ, y ;
    ll_mis_1 = -log(theta[3]) -(x_mis-theta[1])##2 / (2#theta[3]##2) ;

    ecurr = theta[2]+theta[5]#theta[4]/theta[3]#(x_mis-theta[1]) ;
    vcurr = theta[4]##2#(1-theta[5]##2) ;
    range = ecurr-8#sqrt(vcurr)||ecurr+8#sqrt(vcurr) ;

    ll_mis_2 = j(n_mis,1,0) ;

    do i=1 to n_mis ;
      call quad(integral,"integral",range[i,]) ;
      ll_mis_2[i]=log(integral) ;
    end ;

    ll_all=(ll_obs_1+ll_obs_2)//(ll_mis_1+ll_mis_2) ;
    ll=ll_all[+] ;
    return(ll) ;
  finish ;

  x0={0 0 1 1 0 0 0 0} ;
  opt1={1} ;
  con={.  . 1e-10 1e-10 -1 . . .,
       .  .     .     .   1 . . .} ;
  call nlpnrr(rc1,xr1,"likelihood",x0) opt=opt1 blc=con ;
  print xr1 ;

  mux=xr1[1] ; muy=xr1[2] ; vx=xr1[3]##2 ; vy=xr1[4]##2 ; cov=xr1[5]#xr1[3]#xr1[4] ; psi1=xr1[6] ; psi2=xr1[7] ; psi3=xr1[8] ;  
  print mux muy vx vy cov psi1 psi2 psi3 ;

quit ;
