proc format ;
  value misf 1="Missing"
             0="Observed" ;
run ;

data test_mnar_comp ;
  call streaminit(777) ;

  mux=0 ; muy=0 ;
  sx =1 ; sy =1 ;
  rho=0.3 ;

  do i=1 to 100 ;
    x=rand("normal",mux,sx) ;
    y=rand("normal",muy+rho*sy/sx*(x-mux),sqrt((1-rho**2)*sy**2)) ;
    output ;
  end ;
run ;

data test_mnar_miss ;
  set test_mnar_comp ;

  call streaminit(77777) ;

  a=-10 ; b=15 ; c=20 ;

  p=exp(a+b*x+c*y)/(1+exp(a+b*x+c*y)) ;
  miss=rand("bernoulli",p) ;

  if miss=1 then y_o=. ;
  else y_o=y ;
run ;

proc corr data=test_mnar_miss ;
  var x y y_o ;
run ;

proc sgplot data=test_mnar_miss ;
  scatter x=x y=y / group=miss ;
  xaxis values=(-3 to 3) ;
  yaxis values=(-3 to 3) ;
  format miss misf. ;
run ;

proc sgplot data=test_mnar_miss ;
  where miss=0 ;
  scatter x=x y=y / group=miss ;
  xaxis values=(-3 to 3) ;
  yaxis values=(-3 to 3) ;
  format miss misf. ;
run ;

proc mcmc data =test_mnar_miss nbi=5000 nmc=500000 thin=50  seed=777 diagnostics=all plots=all monitor=(mu cov psi) ;
  array yy{2} x y_o ;
  array mu{2} mu1-mu2 ;
  array psi{3} psi1-psi3 ;
  array cov[2,2] ;
  array s0[2,2] (0.0001      0
                      0 0.0001) ;

  parms mu1-mu2 0 ;
  parms psi1-psi3  0 ;
  parms cov ;

  prior mu1-mu2 ~ normal(0,var=10000) ;
  prior psi1-psi3 ~ normal(0,var=10000) ;
  prior cov ~ iwish(3,S0) ;

  p=exp(psi[1]+psi[2]*yy[1]+psi[3]*yy[2]) ;
  pp=p/(1+p) ;

  if miss=0 then ll=log(1-pp) ;
  else ll=log(pp) ;

  model yy ~ mvn(mu,cov) ;
  model general(ll) ;
run ;
